﻿using Plane.Copters;
using GalaSoft.MvvmLight;
using Microsoft.Practices.ServiceLocation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;

namespace Plane.FormationCreator.Formation
{
    public partial class FlightTaskSingleCopterInfo : ObservableObject
    {
        public FlightTaskSingleCopterInfo(ICopter copter)
        {
            this.PropertyChanged += (sender, e) =>
            {
                switch (e.PropertyName)
                {
                    case nameof(TargetLat):
                    case nameof(TargetLng):
                    case nameof(TargetAlt):
                        _flightTaskManager.RaiseSingleCopterTaskChanged(this);
                        break;
                    default:
                        break;
                }
            };

            this.Copter = copter;
        }

        private CopterManager _copterManager = ServiceLocator.Current.GetInstance<CopterManager>();
        private FlightTaskManager _flightTaskManager = ServiceLocator.Current.GetInstance<FlightTaskManager>();

        public ICopter Copter { get; set; }

        private double? _TargetLat;
        public double TargetLat { get { return _TargetLat ?? Copter.Latitude; } set { Set(nameof(TargetLat), ref _TargetLat, value); } }

        private double? _TargetLng;
        public double TargetLng { get { return _TargetLng ?? Copter.Longitude; } set { Set(nameof(TargetLng), ref _TargetLng, value); } }

        private float? _TargetAlt;
        public float TargetAlt { get { return _TargetAlt ?? Copter.Altitude; } set { Set(nameof(TargetAlt), ref _TargetAlt, value); } }

        //应用当前航点高度到所选
        private ICommand _SetAllCopterAltCommand;
        public ICommand SetAllCopterAltCommand
        {
            get
            {
                return _SetAllCopterAltCommand ?? (_SetAllCopterAltCommand = new RelayCommand<double>(async  =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.AcceptingControlCopters.Contains(info.Copter))
                        {
                            info.TargetAlt = TargetAlt;
                        }
                    }  
                }));
            }
        }

        //应用当前是否是降落点到所选
        private ICommand _SetIsLandCommand;
        public ICommand SetIsLandCommand
        {
            get
            {
                return _SetIsLandCommand ?? (_SetIsLandCommand = new RelayCommand<double>(async =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.AcceptingControlCopters.Contains(info.Copter))
                        {
                            info.IsLandWaypoint = IsLandWaypoint;
                        }
                    }
                }));
            }
        }

        //应用当前是否是降落点到所选
        private ICommand _SetIsChangeCommand;
        public ICommand SetIsChangeCommand
        {
            get
            {
                return _SetIsChangeCommand ?? (_SetIsChangeCommand = new RelayCommand<double>(async =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.AcceptingControlCopters.Contains(info.Copter))
                        {
                            info.IsChangeSpeed = IsChangeSpeed;
                            info.LevelSpeed = LevelSpeed;
                            info.UpSpeed = UpSpeed;
                            info.DownSpeed = DownSpeed;
                        }
                    }
                }));
            }
        }


        private ICommand _SetSelLandWaitTimeCommand;
        public ICommand SetSelLandWaitTimeCommand
        {
            get
            {
                return _SetSelLandWaitTimeCommand ?? (_SetSelLandWaitTimeCommand = new RelayCommand<double>(async =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.AcceptingControlCopters.Contains(info.Copter))
                        {
                            info.LandWaitTime = LandWaitTime;
                        }
                    }
                }));
            }
        }

        private ICommand _SetSelTakeOffWaitTimeCommand;
        public ICommand SetSelTakeOffWaitCommand
        {
            get
            {
                return _SetSelTakeOffWaitTimeCommand ?? (_SetSelTakeOffWaitTimeCommand = new RelayCommand<double>(async =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.AcceptingControlCopters.Contains(info.Copter))
                        {
                            info.TakeOffWaitTime = TakeOffWaitTime;
                        }
                    }
                }));
            }
        }


        private int _TakeOffMinTime = 0;
        public int TakeOffMinTime
        {
            get { return _TakeOffMinTime; }
            set { Set(nameof(TakeOffMinTime), ref _TakeOffMinTime, value); }
        }

        private int _TakeOffMaxTime = 10;
        public int TakeOffMaxTime
        {
            get { return _TakeOffMaxTime; }
            set { Set(nameof(TakeOffMaxTime), ref _TakeOffMaxTime, value); }
        }




        private ICommand _SetRandomOffWaitDelayCommand;
        public ICommand SetRandomOffWaitDelayCommand
        {
            get
            {
                return _SetRandomOffWaitDelayCommand ?? (_SetRandomOffWaitDelayCommand = new RelayCommand<double>(async =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.AcceptingControlCopters.Contains(info.Copter))
                        {

                            Random random = new Random(Guid.NewGuid().GetHashCode());
                            ushort delay = (ushort)random.Next(TakeOffMinTime, TakeOffMaxTime);
                            info.TakeOffWaitTime = delay;
                        }
                    }
                }));
            }
        }


        public LatLng LatLngOffset
        {
            get
            {
                //var origin = GetOrigin();
                var origin = new LatLng(_flightTaskManager.OriginLat, _flightTaskManager.OriginLng);
                return new LatLng(TargetLat - origin.Lat, TargetLng - origin.Lng);
            }
            set
            {
                //var origin = GetOrigin();
                var origin = new LatLng(_flightTaskManager.OriginLat, _flightTaskManager.OriginLng);
                TargetLat = origin.Lat + value.Lat;
                TargetLng = origin.Lng + value.Lng;
            }
        }

        public LatLng GetOrigin()
        {
            return _copterManager.Copters.GetCenter() ?? LatLng.Empty;
        }

        public bool isFinished;
        public int takeOffStage; // 判断起飞处于哪个阶段. (0 -> 1 ->2, 对应起飞到10m -> 水平飞行 -> 垂直飞行)
        public int RTLStage;     // 判断降落返航处于哪个阶段. (0->1->2, 对应降落到10m -> 水平飞行 -> 垂直降落)
        public int FlyToStage;   // 判断FlyTo处于哪个阶段
    }
}
