﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace Plane.Util
{
    public class CommNTRIP: IDisposable
    {
        public TcpClient client = new TcpClient();
        private Uri remoteUri;
        double Lat = 40.0910757034397;
        double Lng = 116.27734627295;
        double alt = 0;
        int retrys = 3;
        DateTime _lastnmea = DateTime.MinValue;
        public string Port { get; set; }

        private string host;

        

        private static int bytes = 0;
        private static int bps = 0;
        public int ReadTimeout
        {
            get; 
            set; 
        }

        public CommNTRIP(string url, double lat, double lng)
        {
            remoteUri = new Uri(url);
            this.Lat = lat;
            this.Lng = lng;
            ReadTimeout = 500;
        }

        public void doConnect()
        {
            string usernamePassword = remoteUri.UserInfo;
            string userpass2 = Uri.UnescapeDataString(usernamePassword);
            string auth = "Authorization: Basic " +
                          Convert.ToBase64String(new ASCIIEncoding().GetBytes(userpass2)) + "\r\n";

            if (usernamePassword == "")
                auth = "";

            host = remoteUri.Host;
            Port = remoteUri.Port.ToString();

            client = new TcpClient(host, int.Parse(Port));
            client.Client.IOControl(IOControlCode.KeepAliveValues, TcpKeepAlive(true, 36000000, 3000), null);

            NetworkStream ns = client.GetStream();

            StreamWriter sw = new StreamWriter(ns);
            StreamReader sr = new StreamReader(ns);

            string line = "GET " + remoteUri.PathAndQuery + " HTTP/1.0\r\n"
                          + "User-Agent: NTRIP MissionPlanner/1.0\r\n"
                          + auth
                          + "Connection: close\r\n\r\n";

            sw.Write(line);
            sw.Flush();

            line = sr.ReadLine(); 

            Console.WriteLine(line);
            if (!line.Contains("200"))
            {
                client.Dispose();
                client = new TcpClient();
                throw new Exception("Bad ntrip Responce\n\n" + line);
            }

            SendNMEA();

            VerifyConnected();
        }

        public int Read(byte[] readto, int offset, int length)
        {
            VerifyConnected();

            SendNMEA();

            try
            {
                if (length < 1) { return 0; }

                return client.Client.Receive(readto, offset, length, SocketFlags.Partial);
            }
            catch { throw new Exception("ntrip Socket Closed"); }
        }


        private void SendNMEA()
        {
            if (Lat != 0 || Lng != 0)
            {
                if (_lastnmea.AddSeconds(30) < DateTime.Now)
                {
                    double latdms = (int)Lat + ((Lat - (int)Lat) * .6f);
                    double lngdms = (int)Lng + ((Lng - (int)Lng) * .6f);

                    var line = string.Format(System.Globalization.CultureInfo.InvariantCulture,
                        "$GP{0},{1:HHmmss.ff},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11},{12},{13},{14}", "GGA",
                        DateTime.Now.ToUniversalTime(), Math.Abs(latdms * 100).ToString("0000.00"), Lat < 0 ? "S" : "N",
                        Math.Abs(lngdms * 100).ToString("00000.00"), Lng < 0 ? "W" : "E", 1, 10,
                        1, alt.ToString("0.00"), "M", 0, "M", "0.0", "0");

                    string checksum = GetChecksum(line);
                    WriteLine(line + "*" + checksum);

                    _lastnmea = DateTime.Now;
                }
            }
        }

        public int BytesToRead
        {
            get
            {
                /*Console.WriteLine(DateTime.Now.Millisecond + " tcp btr " + (client.Available + rbuffer.Length - rbufferread));*/
                SendNMEA();
                return (int)client.Available;
            }
        }

        void VerifyConnected()
        {
            if (!IsOpen)
            {
                try
                {
                    client.Dispose();
                    client = new TcpClient();
                }
                catch { }

                // this should only happen if we have established a connection in the first place
                if (client != null && retrys > 0)
                {
                    doConnect();
                    retrys--;
                }

                throw new Exception("The ntrip is closed");
            }
        }

        public bool IsOpen
        {
            get
            {
                try
                {
                    return client.Client.Connected;
                }
                catch
                {
                    return false;
                }
            }
        }

        public void WriteLine(string line)
        {
            VerifyConnected();
            line = line + "\r\n";
            Write(line);
        }

        public void Write(string line)
        {
            VerifyConnected();
            byte[] data = new System.Text.ASCIIEncoding().GetBytes(line);
            Write(data, 0, data.Length);
        }

        public void Write(byte[] write, int offset, int length)
        {
            VerifyConnected();
            try
            {
                client.Client.Send(write, length, SocketFlags.None);
            }
            catch { }//throw new Exception("Comport / Socket Closed"); }
        }

        string GetChecksum(string sentence)
        {
            // Loop through all chars to get a checksum
            int Checksum = 0;
            foreach (char Character in sentence.ToCharArray())
            {
                switch (Character)
                {
                    case '$':
                        // Ignore the dollar sign
                        break;
                    case '*':
                        // Stop processing before the asterisk
                        continue;
                    default:
                        // Is this the first value for the checksum?
                        if (Checksum == 0)
                        {
                            // Yes. Set the checksum to the value
                            Checksum = Convert.ToByte(Character);
                        }
                        else
                        {
                            // No. XOR the checksum with this character's value
                            Checksum = Checksum ^ Convert.ToByte(Character);
                        }
                        break;
                }
            }
            // Return the checksum formatted as a two-character hexadecimal
            return Checksum.ToString("X2");
        }

        private byte[] TcpKeepAlive(bool On_Off, uint KeepaLiveTime, uint KeepaLiveInterval)
        {
            byte[] InValue = new byte[12];

            Array.ConstrainedCopy(BitConverter.GetBytes(Convert.ToUInt32(On_Off)), 0, InValue, 0, 4);
            Array.ConstrainedCopy(BitConverter.GetBytes(KeepaLiveTime), 0, InValue, 4, 4);
            Array.ConstrainedCopy(BitConverter.GetBytes(KeepaLiveInterval), 0, InValue, 8, 4);

            return InValue;
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                this.Close();
                client = null;
            }
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        public void Close()
        {
            try
            {
                if (client.Client != null && client.Client.Connected)
                {
                    client.Client.Dispose();
                    client.Dispose();
                }
            }
            catch { }

            try
            {
                client.Dispose();
            }
            catch { }

            client = new TcpClient();
        }   
    }
}
